@@ -1,5 +1,6 @@
 # -*- coding: utf-8 -*-
 from AccessControl import Unauthorized
+from datetime import datetime
 from plone.app.theming.interfaces import _
 from plone.app.theming.interfaces import DEFAULT_THEME_FILENAME
 from plone.app.theming.interfaces import IThemeSettings
@@ -19,8 +20,8 @@ from plone.memoize.instance import memoize
 from plone.registry.interfaces import IRegistry
 from plone.resource.utils import queryResourceDirectory
 from Products.CMFCore.utils import getToolByName
+from Products.CMFPlone.utils import safe_unicode
 from Products.CMFPlone.interfaces import ILinkSchema
-from Products.Five.browser.decode import processInputs
 from Products.statusmessages.interfaces import IStatusMessage
 from zope.component import getMultiAdapter
 from zope.component import getUtility
@@ -29,9 +30,18 @@ from zope.publisher.browser import BrowserView
 from zope.schema.interfaces import IVocabularyFactory
 
 import logging
+import six
 import zipfile
 
 
+try:
+    # Zope 4
+    from Products.Five.browser.decode import processInputs
+except ImportError:
+    # Zope 5
+    processInputs = None
+
+
 logger = logging.getLogger('plone.app.theming')
 
 
@@ -52,7 +62,6 @@ class ThemingControlpanel(BrowserView):
 
     def __call__(self):
         self.pskin = getToolByName(self.context, 'portal_skins')
-
         if self.update():
             return self.index()
         return ''
@@ -105,7 +114,8 @@ class ThemingControlpanel(BrowserView):
 
     def update(self):
         # XXX: complexity too high: refactoring needed
-        processInputs(self.request)
+        if processInputs is not None:
+            processInputs(self.request)
         self._setup()
         self.errors = {}
         form = self.request.form
@@ -188,6 +198,8 @@ class ThemingControlpanel(BrowserView):
             markSpecialLinks = form.get('markSpecialLinks', None)
             extLinksOpenInNewWindow = form.get('extLinksOpenInNewWindow', None)
 
+            custom_css = form.get('custom_css', b'')
+
             if not self.errors:
                 # Trigger onDisabled() on plugins if theme was active
                 # previously and rules were changed
@@ -199,11 +211,16 @@ class ThemingControlpanel(BrowserView):
                 self.theme_settings.absolutePrefix = prefix
                 self.theme_settings.parameterExpressions = parameterExpressions
                 self.theme_settings.hostnameBlacklist = hostnameBlacklist
+                if custom_css != self.theme_settings.custom_css:
+                    self.theme_settings.custom_css_timestamp = datetime.now()
+                self.theme_settings.custom_css = custom_css
                 self.theme_settings.doctype = doctype
 
                 # Theme base settings
                 if themeBase is not None:
-                    self.pskin.default_skin = themeBase.encode('utf-8')
+                    if six.PY2:
+                        themeBase = themeBase.encode('utf-8')
+                    self.pskin.default_skin = themeBase
                 if markSpecialLinks is not None:
                     self.mark_special_links = markSpecialLinks
                 if extLinksOpenInNewWindow is not None:
@@ -433,14 +450,14 @@ class ThemingControlpanel(BrowserView):
             override = False
 
             # Is there more than one theme with the same name?
-            if len(filter(lambda x: x.__name__ == theme.__name__, self.availableThemes)) > 1:
+            if len([x for x in self.availableThemes if x.__name__ == theme.__name__]) > 1:
                 # Then we make sure we're using the TTW version, not the filesystem version.
                 try:
-                    theme = filter(lambda x: x.__name__ == theme.__name__, self.zodbThemes)[0]
+                    theme = list(filter(lambda x: x.__name__ == theme.__name__, self.zodbThemes))[0]
                     override = True
                 # Or when TTW is not available, the first available filesystem version.
                 except IndexError:
-                    theme = filter(lambda x: x.__name__ == theme.__name__, self.availableThemes)[0]
+                    theme = list(filter(lambda x: x.__name__ == theme.__name__, self.availableThemes))[0]
 
             previewUrl = "++resource++plone.app.theming/defaultPreview.png"
             if theme.preview:
