@@ -23,6 +23,8 @@ from zope.component import getUtility
 
 import os.path
 import re
+import six
+import transaction
 import unittest
 
 
@@ -45,7 +47,6 @@ class TestCase(unittest.TestCase):
             'requestParam': 'request/useother | string:off',
         }
 
-        import transaction
         transaction.commit()
 
     def tearDown(self):
@@ -77,7 +78,6 @@ class TestCase(unittest.TestCase):
         portal = self.layer['portal']
 
         self.settings.enabled = True
-        import transaction
         transaction.commit()
 
         browser = Browser(app)
@@ -140,7 +140,6 @@ class TestCase(unittest.TestCase):
             self.settings.doctype,
             u"<!DOCTYPE html>"
         )
-        import transaction
         transaction.commit()
 
         browser = Browser(app)
@@ -163,7 +162,6 @@ class TestCase(unittest.TestCase):
         portal = self.layer['portal']
 
         self.settings.enabled = True
-        import transaction
         transaction.commit()
 
         browser = Browser(app)
@@ -185,7 +183,6 @@ class TestCase(unittest.TestCase):
         getConfiguration().debug_mode = False
 
         self.settings.enabled = True
-        import transaction
         transaction.commit()
 
         browser = Browser(app)
@@ -205,7 +202,6 @@ class TestCase(unittest.TestCase):
         portal = self.layer['portal']
 
         self.settings.enabled = True
-        import transaction
         transaction.commit()
 
         browser = Browser(app)
@@ -241,20 +237,16 @@ class TestCase(unittest.TestCase):
         portal = self.layer['portal']
 
         # We'll upload the theme files to the Plone site root
-        rules_contents = open(
-            os.path.join(os.path.split(__file__)[0], 'localrules.xml')
-        )
-        theme_contents = open(
-            os.path.join(os.path.split(__file__)[0], 'theme.html')
-        )
-        portal.manage_addDTMLMethod('theme.html', file=theme_contents)
-        portal.manage_addDTMLMethod('rules.xml', file=rules_contents)
+        here = os.path.split(__file__)[0]
+        with open(os.path.join(here, 'localrules.xml')) as rules_contents:
+            portal.manage_addDTMLMethod('rules.xml', file=rules_contents)
+        with open(os.path.join(here, 'theme.html')) as theme_contents:
+            portal.manage_addDTMLMethod('theme.html', file=theme_contents)
 
         # These paths should be relative to the Plone site root
         self.settings.rules = u'/rules.xml'
         self.settings.enabled = True
 
-        import transaction
         transaction.commit()
 
         browser = Browser(app)
@@ -274,14 +266,11 @@ class TestCase(unittest.TestCase):
         portal = self.layer['portal']
 
         # We'll upload the theme files to the Plone site root
-        rules_contents = open(
-            os.path.join(os.path.dirname(__file__), 'localrules.xml')
-        )
-        theme_contents = open(
-            os.path.join(os.path.dirname(__file__), 'theme.html')
-        )
-        portal.manage_addDTMLMethod('theme.html', file=theme_contents)
-        portal.manage_addDTMLMethod('rules.xml', file=rules_contents)
+        here = os.path.split(__file__)[0]
+        with open(os.path.join(here, 'localrules.xml')) as rules_contents:
+            portal.manage_addDTMLMethod('rules.xml', file=rules_contents)
+        with open(os.path.join(here, 'theme.html')) as theme_contents:
+            portal.manage_addDTMLMethod('theme.html', file=theme_contents)
 
         # These paths should be relative to the Plone site root
         self.settings.rules = u'/rules.xml'
@@ -290,7 +279,6 @@ class TestCase(unittest.TestCase):
         from Products.SiteAccess import VirtualHostMonster
         VirtualHostMonster.manage_addVirtualHostMonster(app, 'virtual_hosting')
 
-        import transaction
         transaction.commit()
 
         portalURL = portal.absolute_url()
@@ -321,7 +309,6 @@ class TestCase(unittest.TestCase):
         self.settings.enabled = True
         self.settings.absolutePrefix = None
 
-        import transaction
         transaction.commit()
 
         browser = Browser(app)
@@ -336,7 +323,6 @@ class TestCase(unittest.TestCase):
         self.settings.enabled = True
         self.settings.absolutePrefix = u'http://example.com'
 
-        import transaction
         transaction.commit()
 
         browser = Browser(app)
@@ -354,7 +340,6 @@ class TestCase(unittest.TestCase):
         self.settings.enabled = True
         self.settings.absolutePrefix = u'/foo'
 
-        import transaction
         transaction.commit()
 
         browser = Browser(app)
@@ -372,7 +357,6 @@ class TestCase(unittest.TestCase):
         from Products.SiteAccess import VirtualHostMonster
         VirtualHostMonster.manage_addVirtualHostMonster(app, 'virtual_hosting')
 
-        import transaction
         transaction.commit()
 
         self.settings.enabled = True
@@ -387,7 +371,6 @@ class TestCase(unittest.TestCase):
             "/_vh_fizz/_vh_buzz/_vh_fizzbuzz/".format(prefix, suffix)
         )
 
-        import transaction
         transaction.commit()
 
         browser = Browser(app)
@@ -406,7 +389,6 @@ class TestCase(unittest.TestCase):
         self.settings.enabled = True
         self.settings.rules = u"invalid"
 
-        import transaction
         transaction.commit()
 
         browser = Browser(app)
@@ -427,13 +409,12 @@ class TestCase(unittest.TestCase):
 
         self.settings.enabled = True
 
-        import transaction
         transaction.commit()
 
         browser = Browser(app)
         browser.open(portal.absolute_url() + '/document_icon.png')
         # The theme
-        self.assertFalse("This is the theme" in browser.contents)
+        self.assertFalse(b"This is the theme" in browser.contents)
 
     # XXX: This relies on a _v_ attribute; the test is too brittle
     #
@@ -456,7 +437,7 @@ class TestCase(unittest.TestCase):
     #         self.settings, portal.absolute_url()
     #     ).updateTransform(transform)
     #
-    #     import transaction; transaction.commit()
+    #     transaction.commit()
     #
     #     browser = Browser(app)
     #     browser.open(portal.absolute_url())
@@ -475,7 +456,7 @@ class TestCase(unittest.TestCase):
     #     self.settings.enabled = False
     #     self.settings.enabled = True
     #
-    #     import transaction; transaction.commit()
+    #     transaction.commit()
     #
     #     browser.open(portal.absolute_url())
     #
@@ -524,7 +505,7 @@ class TestCase(unittest.TestCase):
     #     # First try without the theme
     #     self.settings.enabled = False
 
-    #     import transaction; transaction.commit()
+    #     transaction.commit()
 
     #     browser = Browser(app)
     #     browser.open(portal.absolute_url())
@@ -540,7 +521,7 @@ class TestCase(unittest.TestCase):
     #     # Now enable the theme and try again
     #     self.settings.enabled = True
 
-    #     import transaction; transaction.commit()
+    #     transaction.commit()
 
     #     browser = Browser(app)
     #     browser.open(portal.absolute_url())
@@ -565,7 +546,6 @@ class TestCase(unittest.TestCase):
 
         self.settings.enabled = True
 
-        import transaction
         transaction.commit()
 
         browser = Browser(app)
@@ -604,7 +584,6 @@ class TestCase(unittest.TestCase):
             'requestParam': 'request/someParam | string:off',
         }
 
-        import transaction
         transaction.commit()
 
         browser = Browser(app)
@@ -648,7 +627,6 @@ class TestCase(unittest.TestCase):
 
         self.settings.enabled = True
 
-        import transaction
         transaction.commit()
 
         browser = Browser(app)
@@ -676,7 +654,6 @@ class TestCase(unittest.TestCase):
             'requestParam': 'request/someParam | string:off',
         }
 
-        import transaction
         transaction.commit()
 
         browser = Browser(app)
@@ -725,7 +702,6 @@ class TestCase(unittest.TestCase):
             'navigation_root_id': 'python:portal_state.navigation_root().getId()'  # noqa
         }
 
-        import transaction
         transaction.commit()
 
         browser = Browser(app)
@@ -778,7 +754,7 @@ class TestCase(unittest.TestCase):
 
     #     self.settings.enabled = True
 
-    #     import transaction; transaction.commit()
+    #     transaction.commit()
 
     #     browser = Browser(app)
 
@@ -800,28 +776,26 @@ class TestCase(unittest.TestCase):
 
         setRoles(portal, TEST_USER_ID, ('Manager',))
 
-        one = open(os.path.join(os.path.split(__file__)[0], 'one.html'))
-        two = open(os.path.join(os.path.split(__file__)[0], 'two.html'))
-
         # Create some test content in the portal root
-        portal.manage_addDTMLMethod('alpha', file=one)
-        portal.manage_addDTMLMethod('beta', file=two)
-
-        one.seek(0)
-        two.seek(0)
+        here = os.path.split(__file__)[0]
+        with open(os.path.join(here, 'one.html')) as one:
+            portal.manage_addDTMLMethod('alpha', file=one)
+        with open(os.path.join(here, 'two.html')) as two:
+            portal.manage_addDTMLMethod('beta', file=two)
 
         # Create some different content in a subfolder
         portal.invokeFactory('Folder', 'subfolder')
         portal.portal_workflow.doActionFor(portal.subfolder, 'publish')
 
-        portal['subfolder'].manage_addDTMLMethod('alpha', file=two)
-        portal['subfolder'].manage_addDTMLMethod('beta', file=one)
+        with open(os.path.join(here, 'one.html')) as one:
+            portal['subfolder'].manage_addDTMLMethod('beta', file=one)
+        with open(os.path.join(here, 'two.html')) as two:
+            portal['subfolder'].manage_addDTMLMethod('alpha', file=two)
 
         # Set up transformation
         self.settings.rules = u'python://plone.app.theming/tests/includes.xml'
         self.settings.enabled = True
 
-        import transaction
         transaction.commit()
 
         browser = Browser(app)
@@ -840,6 +814,44 @@ class TestCase(unittest.TestCase):
         self.assertTrue('<div id="alpha">Number one</div>' in browser.contents)
         self.assertTrue('<div id="beta">Number one</div>' in browser.contents)
 
+    def test_include_non_ascii(self):
+        # A Diazo theme with non-ascii subrequest can show characters wrong with Python 3.
+        # See https://github.com/plone/Products.CMFPlone/issues/3068
+        # These are the same French characters:
+        # 'Actualités'
+        # b'Actualit\xc3\xa9s'
+        # u'Actualit\xe9s'
+        # 'Actualit&#195;&#169;s'
+
+        app = self.layer['app']
+        portal = self.layer['portal']
+
+        setRoles(portal, TEST_USER_ID, ('Manager',))
+
+        # Create some test content in the portal root
+        here = os.path.split(__file__)[0]
+        with open(os.path.join(here, 'french.html'), "rb") as french:
+            portal.manage_addDTMLMethod('french', file=french)
+
+        # Set up transformation
+        self.settings.rules = u'python://plone.app.theming/tests/nonascii.xml'
+        self.settings.enabled = True
+
+        transaction.commit()
+
+        browser = Browser(app)
+
+        # At the root if the site, we should pick up 'one' for alpha (absolute
+        # path, relative to site root) and 'two' for beta (relative path,
+        # relative to current directory)
+
+        browser.open(portal.absolute_url())
+        # browser.contents is always string.  On Py 2 this means bytes, on Py 3 text.
+        if six.PY2:
+            self.assertIn(b'<div id="content">Actualit\xc3\xa9s</div>', browser.contents)
+        else:
+            self.assertIn(u'<div id="content">Actualit\xe9s</div>', browser.contents)
+
     def test_css_js_includes(self):
 
         app = self.layer['app']
@@ -847,7 +859,6 @@ class TestCase(unittest.TestCase):
 
         self.settings.enabled = True
         self.settings.rules = u'/++theme++plone.app.theming.tests/css-js.xml'
-        import transaction
         transaction.commit()
 
         browser = Browser(app)
@@ -870,22 +881,23 @@ class TestCase(unittest.TestCase):
 
         self.settings.enabled = True
         self.settings.rules = u'/++theme++plone.app.theming.tests/nonascii.xml'
-        import transaction
         transaction.commit()
 
         browser = Browser(app)
         browser.open(portal.absolute_url())
 
-        self.assertTrue(
-            '''<div>N\xc3\xbamero uno</div>'''
-            in browser.contents)
+        # browser.contents is always string.  On Py 2 this means bytes, on Py 3 text.
+        if six.PY2:
+            self.assertIn(b'<div>N\xc3\xbamero uno</div>', browser.contents)
+        else:
+            self.assertIn(u'<div>N\xfamero uno</div>', browser.contents)
+
 
     def test_theme_enabled_query_string_debug_switch(self):
         app = self.layer['app']
         portal = self.layer['portal']
 
         self.settings.enabled = True
-        import transaction
         transaction.commit()
 
         browser = Browser(app)
